import { IChangePassword, IConf, IContact, ICreateGroup, ICreateGroupResp, ICreateGroupUser, ICreateSecret, ICreateSecretThroughGroup, ICreateUser, IGroup, IGroupUser, ILoginResonse, IPublicAuthResponse, ISecret, ISecretThroughGroup, IUpdateSecret, IUpdateSecretWithThroughs, IUser, IUserPublicKey } from "./api.interfaces";
export interface IHeaders {
    [propName: string]: string;
}
export default class Api {
    baseUrl: string;
    token: string;
    constructor(baseUrl?: string);
    login(base: string, expires?: number): Promise<ILoginResonse>;
    logout(): Promise<void>;
    logoutAll(): Promise<void>;
    usernameAvailable(username: string): Promise<{
        available: boolean;
    }>;
    lookupUserID(email: string): Promise<{
        id: number;
    }>;
    createUser(user: ICreateUser): Promise<IUser>;
    updateUser(id: any, user: Partial<ICreateUser>): Promise<IUser>;
    getUser(): Promise<IUser>;
    deleteUser(userId: number, password: string): Promise<void>;
    getPublicAuth(username: string): Promise<IPublicAuthResponse>;
    getUserPublicKey(id: number): Promise<IUserPublicKey>;
    createGroup(group: ICreateGroup): Promise<ICreateGroupResp>;
    updateGroup(id: number, group: Partial<ICreateGroup>): Promise<ICreateGroupResp>;
    deleteGroup(id: number): Promise<void>;
    getGroup(id: number): Promise<IGroup>;
    getGroups(): Promise<IGroup[]>;
    createGroupUser(groupID: number, groupUser: ICreateGroupUser): Promise<IGroupUser>;
    deleteGroupUser(groupID: number, id: number): Promise<void>;
    acceptGroupInvite(groupUserId: number): Promise<{}>;
    declineGroupInvite(groupUserId: number): Promise<{}>;
    getContacts(): Promise<IContact[]>;
    createSecret(secret: ICreateSecret): Promise<ISecret>;
    updateSecret(id: number, secret: IUpdateSecret): Promise<ISecret>;
    updateSecretWithThroughs(id: number, secret: IUpdateSecretWithThroughs): Promise<ISecret>;
    getSecret(id: number): Promise<ISecret>;
    getSecrets(): Promise<ISecret[]>;
    deleteSecret(id: number): Promise<void>;
    createSecretGroup(secretID: number, secretGroup: ICreateSecretThroughGroup): Promise<ISecretThroughGroup>;
    deleteSecretGroup(secretID: number, secretThroughID: number): Promise<void>;
    changePassword(changePassword: IChangePassword): Promise<{
        success: boolean;
    }>;
    confirmEmailLongCode(code: string): Promise<{
        user_id: number;
    }>;
    confirmEmailShortCode(code: string): Promise<{
        user_id: number;
    }>;
    requestNewConfirmation(): Promise<void>;
    getConf(): Promise<IConf>;
    protected sendInner<B>(method: string, url: string, body: B, headers: IHeaders, returnsJSON: boolean): Promise<any>;
    private get<R>(url);
    private post<B, R>(url, body?, headers?);
    private patch<B, R>(url, body);
    private put<B, R>(url, body);
    private delete<B>(url, body?);
    private send<B>(method, url, {body, returnsJSON, headers}?);
}
