import { KeyPair } from "simple-asymmetric-js/js/asymmetric_encryption";
import Api from "./api";
import * as api from "./api.interfaces";
import { INewSecret, ISecret } from "./sdk.interfaces";
export default class PassitSDK {
    userId: number;
    api: Api;
    keyPair: KeyPair;
    constructor();
    set_up(publicKey: string, privateKey: string, userId: number, authToken: string): Promise<void>;
    set_url(url: string): void;
    ready(): Promise<void>;
    set_keys(publicKey: string, privateKey: string): void;
    get_conf(): Promise<api.IConf>;
    is_username_available(email: string): Promise<boolean>;
    sign_up(email: string, password: string, firstName?: string, lastName?: string): Promise<{
        backupCode: string;
        user: api.IUser;
        token: string;
        privateKey: string;
    }>;
    log_in(email: string, password: string, expires?: number): Promise<{
        user: api.IUser;
        token: string;
        privateKey: string;
    }>;
    logout(): Promise<void>;
    logoutAll(): Promise<void>;
    update_user(id: number, user: Partial<api.ICreateUser>): Promise<api.IUser>;
    confirm_email_long_code(code: string): Promise<number>;
    confirm_email_short_code(code: string): Promise<number>;
    request_new_confirmation(): Promise<void>;
    lookup_user_id(email: string): Promise<number>;
    deleteOwnAccount(password: string): Promise<void>;
    create_group(name: string, slug?: string): Promise<api.ICreateGroupResp>;
    update_group({id, name, slug}: {
        id: number;
        name: string;
        slug: string;
    }): Promise<api.ICreateGroupResp>;
    delete_group(groupId: number): Promise<void>;
    get_group(groupID: number): Promise<api.IGroup>;
    acceptGroupInvite(groupUserId: number): Promise<{}>;
    declineGroupInvite(groupUserId: number): Promise<{}>;
    list_groups(): Promise<api.IGroup[]>;
    add_user_to_group(groupID: number, userID: number, email: string): Promise<api.IGroupUser>;
    remove_user_from_group(groupId: number, groupUserId: number): Promise<void>;
    list_contacts(): Promise<api.IContact[]>;
    create_secret(secret: INewSecret): Promise<api.ISecret>;
    update_secret(secret: ISecret): Promise<api.ISecret>;
    delete_secret(secretID: number): Promise<void>;
    get_secret(secretID: number): Promise<api.ISecret>;
    list_secrets(): Promise<api.ISecret[]>;
    decrypt_secret(uniqueSecret: api.ISecret): Promise<api.IData>;
    offline_decrypt_secret(keyCiphertext: string, data: api.IData, group?: api.IGroup): api.IData;
    add_group_to_secret(groupId: number, secret: number | api.ISecret): Promise<api.ISecretThroughGroup>;
    remove_group_from_secret(secretId: number, secretThroughId: number): Promise<void>;
    change_password(oldPassword: string, newPassword: string): Promise<void>;
    reencryptSecretThroughSet(userKeyPair: KeyPair): Promise<api.IChangePasswordSecretThrough[]>;
    reencryptGroupUserSet(userKeyPair: KeyPair): Promise<api.IChangePasswordGroupUser[]>;
    private _check_if_authenticated();
    private _decrypt_secrets(encryptedKey, encryptedSecret, keyPair?);
    private _getGroupKeyPair(group);
    private generateGroupKey(groupPrivateKey, userPublicKey);
    private _encrypt_secrets(secrets, publicKey?);
    private do_log_in(email, hash, password, expires?);
}
