"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var fetchMock = require("fetch-mock");
var exceptions_1 = require("./exceptions");
var sdk_1 = require("./sdk");
var testResponses_1 = require("./testResponses");
var email = testResponses_1.clientEmail;
var testPassword = "aaaaaaaa";
jasmine.DEFAULT_TIMEOUT_INTERVAL = 10000;
var forceLogin = function (sdk, realLogin, password) {
    if (realLogin === void 0) { realLogin = false; }
    if (password === void 0) { password = testPassword; }
    return new Promise(function (resolve, reject) {
        if (realLogin === false) {
            fetchMock
                .mock("end:auth/login/", testResponses_1.loginResp)
                .mock("end:user-public-auth/" + email + "/", testResponses_1.publicUser);
        }
        sdk.log_in(email, password).then(function (resp) {
            resolve();
        });
    });
};
describe("PassitSDK class can", function () {
    var sdk;
    beforeEach(function () {
        sdk = new sdk_1.default();
    });
    afterEach(function () {
        fetchMock.restore();
    });
    it("check username availability", function (done) {
        fetchMock
            .mock("end:username-available/" + email + "/", { available: true });
        sdk.is_username_available(email).then(function (isAvailable) {
            expect(isAvailable).toBe(true);
            done();
        });
        var email2 = "test2@test.com";
        fetchMock
            .mock("end:username-available/" + email2 + "/", { available: false });
        sdk.is_username_available(email2).then(function (isAvailable) {
            expect(isAvailable).toBe(false);
            done();
        });
    });
    it("sign up", function (done) {
        var password = testPassword;
        fetchMock
            .mock("end:users/", testResponses_1.signUpResp)
            .mock("end:auth/login/", testResponses_1.loginResp);
        sdk.sign_up(email, password).then(function (resp) {
            expect(resp.user.email).toBe(email);
            done();
        });
    });
    it("log in", function (done) {
        var password = testPassword;
        fetchMock
            .mock("end:user-public-auth/" + email + "/", testResponses_1.publicUser)
            .mock("end:auth/login/", testResponses_1.loginResp);
        sdk.log_in(email, password).then(function (resp) {
            expect(resp).toBeTruthy();
            done();
        });
    });
    it("log in but server is down", function (done) {
        var password = testPassword;
        fetchMock
            .mock("end:user-public-auth/" + email + "/", {})
            .mock("end:auth/login/", { throws: "error" });
        sdk.log_in(email, password).catch(function (err) {
            expect(err).toBeTruthy();
            done();
        });
    });
    it("deny incorrect log in", function (done) {
        var password = "1234567";
        fetchMock
            .mock("end:user-public-auth/" + email + "/", testResponses_1.publicUser)
            .mock("end:auth/login/", {
            status: 401,
            body: { detail: "Invalid username/password." },
        });
        sdk.log_in(email, password).then(function (resp) {
            fail("Should not have been able to log in");
        }).catch(function (ex) {
            expect(ex).not.toBe(null);
            done();
        });
    });
    it("create group", function (done) {
        fetchMock
            .post("end:groups/", {
            status: 201,
            body: { id: 1, name: "mygroup", slug: "mygroup" },
        });
        forceLogin(sdk).then(function () {
            var group = "mygroup";
            sdk.create_group(group, group).then(function (resp) {
                expect(resp.name).toBe(group);
                done();
            });
        });
    });
    it("create group and generate slug", function (done) {
        fetchMock
            .post("end:groups/", {
            status: 201,
            body: { id: 1, name: "My Group", slug: "my-group" },
        });
        forceLogin(sdk).then(function () {
            var group = "My Group";
            sdk.create_group(group).then(function (resp) {
                expect(resp.name).toBe(group);
                done();
            });
        });
    });
    it("update existing group", function (done) {
        var groupResp1 = '{"id":38,"name":"My Group","slug":"my-group"}';
        var groupResp2 = '{"id":38,"name":"My Edited Group","slug":"my-group"}';
        fetchMock
            .post("end:groups/", { status: 201, body: groupResp1 })
            .patch("end:groups/38/", groupResp2);
        var group = "My Group";
        forceLogin(sdk).then(function () {
            sdk.create_group(group).then(function (newGroup) {
                var newName = "My Edited Group";
                newGroup.name = newName;
                sdk.update_group(newGroup).then(function (updatedGroup) {
                    expect(updatedGroup.name).toBe(newName);
                    done();
                });
            });
        });
    });
    it("fail to create group and when not logged in", function () {
        var group = "My Group";
        sdk.create_group(group).catch(function (err) {
            expect(err).toEqual(new exceptions_1.AuthenticationRequiredException("create_group"));
        });
    });
    it("add user to group", function (done) {
        var groupId = 1;
        var userId = 2;
        fetchMock
            .get("end:groups/" + groupId + "/", testResponses_1.groupDetailResp)
            .get("end:user-public-key/" + userId + "/", testResponses_1.userPublicKeyResp)
            .post("end:groups/" + groupId + "/users/", testResponses_1.userAddGroupResp);
        forceLogin(sdk).then(function () {
            sdk.add_user_to_group(groupId, userId, "test@example.com").then(function (resp) {
                expect(resp.user).toBe(userId);
                expect(resp.group).toBe(groupId);
                done();
            });
        });
    });
    it("can list contacts", function (done) {
        var contact = {
            id: 5,
            email: "test@example.com",
            first_name: "My",
            last_name: "Friend",
        };
        fetchMock
            .get("end:contacts/", [contact]);
        forceLogin(sdk).then(function () {
            sdk.list_contacts().then(function (contacts) {
                expect(contacts[0].id).toBe(5);
                done();
            });
        });
    });
    it("remove user from group", function (done) {
        var groupId = 1;
        var groupUserId = 1;
        fetchMock
            .delete("end:groups/" + groupId + "/users/" + groupUserId + "/", { status: 204 });
        forceLogin(sdk).then(function () {
            sdk.remove_user_from_group(groupId, groupUserId).then(function () {
                done();
            });
        });
    });
    it("delete groups", function (done) {
        fetchMock
            .post("end:groups/", { id: 52, name: "Test Group", slug: "test-group" })
            .delete("end:groups/52/", { status: 204 });
        forceLogin(sdk).then(function () {
            sdk.create_group("Test Group").then(function (group) {
                sdk.delete_group(group.id).then(function () {
                    done();
                });
            });
        });
    });
    it("get group", function (done) {
        var groupId = 7;
        fetchMock
            .get("end:groups/" + groupId + "/", testResponses_1.groupResp);
        forceLogin(sdk).then(function () {
            sdk.get_group(groupId).then(function (resp) {
                expect(resp.name).toBe("devs");
                done();
            });
        });
    });
    it("list groups", function (done) {
        fetchMock
            .get("end:groups/", [testResponses_1.groupResp]);
        forceLogin(sdk).then(function () {
            sdk.list_groups().then(function (resp) {
                expect(resp[0].name).toBe("devs");
                done();
            });
        });
    });
    it("create secret", function (done) {
        var name = "my secret";
        var type = "website";
        var visibleData = {
            username: "myself",
        };
        var secrets = {
            password: "hunter2",
        };
        fetchMock
            .post("end:secrets/", testResponses_1.createSecretResp);
        forceLogin(sdk).then(function () {
            sdk.create_secret({
                name: name,
                type: type,
                visible_data: visibleData,
                secrets: secrets,
            }).then(function (resp) {
                expect(resp.name).toBe(name);
                expect(resp.type).toBe(type);
                done();
            });
        });
    });
    it("create secret handles errors", function (done) {
        var name = "";
        var type = "website";
        var visibleData = {
            username: "",
        };
        forceLogin(sdk).then(function () {
            sdk.create_secret({
                name: name,
                type: type,
                visible_data: visibleData,
            }).catch(function (err) {
                expect(err).toBeTruthy();
                done();
            });
        });
    });
    it("raises nice errors on unauthed create secret", function () { return __awaiter(_this, void 0, void 0, function () {
        var name, type, visibleData, secrets, data, err_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    name = "my secret";
                    type = "website";
                    visibleData = {
                        username: "myself",
                    };
                    secrets = {
                        password: "hunter2",
                    };
                    data = {
                        name: name,
                        type: type,
                        visible_data: visibleData,
                        secrets: secrets,
                    };
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4, sdk.create_secret(data)];
                case 2:
                    _a.sent();
                    return [3, 4];
                case 3:
                    err_1 = _a.sent();
                    expect(err_1).toEqual(new exceptions_1.AuthenticationRequiredException("create_secret"));
                    return [3, 4];
                case 4: return [2];
            }
        });
    }); });
    it("create group secret", function (done) {
        var name = "my group secret";
        var type = "website";
        var visibleData = {
            username: "myself",
        };
        var secrets = {
            password: "hunter2",
        };
        var groupId = 7;
        var groupName = "My Group";
        var publicKey = "xRhhY/M0r65jDMRjoMVcgHEdZyj1EXGDyLGXzPYByQI=";
        var myKeyCiphertext = "Ib1dn1oOWxCHsUqvCfpOzMThY1fmFVYyy9O+4cokXBXMsLbkPTu7inPvf66iB/TEqld31O7ubOBTUoCvfKUpNCIOVxzUOGwC964DcNsdeL61IvkhOSrvmroP/Yk=";
        var myPrivateKeyCiphertext = "zSxd4cJX5MyyHvhU/nJvvEzb9fu1WASm9t4UybiLrtyzmpFzfUPYtPxEd7rZkPnDu/NI5UB8UwSrhfLc+huCfOvYfDII2lCbKfPNYsUruztrYgTk";
        var password = "ypXHWRqYEuWb7eNvdTOtHRCjzqxaVH3/QJuKWRy+DyPKRFGKn1LYQzGxr1GnTOo=";
        var keyCiphertext = "T1vB9Omm7lLs1w4IcxgxHG1G6lZxeu61ob5uiLn6rWXET7bIiUgvRMWqvZua/JR/DxJWEfxcKJIcpBRsSIYMI7Ga4QT7kI+96cpdNEGdUJ3F7UhkH/EqbbgZeSM=";
        var groupPostResp = { id: 7, name: "My Group", slug: "my-group" };
        var groupGetResp1 = { id: 7, name: "My Group", groupuser_set: [{ id: 7, user: 1, group: 7, is_group_admin: true }], public_key: publicKey, my_key_ciphertext: myKeyCiphertext, my_private_key_ciphertext: myPrivateKeyCiphertext };
        var secretPostResp = { id: 8, name: "my group secret", type: "website", data: { username: "myself" }, secret_through_set: [{ id: 8, group: 7, key_ciphertext: keyCiphertext, data: { password: password }, public_key: publicKey, is_mine: true }] };
        fetchMock
            .post("end:groups/", groupPostResp)
            .get("end:groups/" + groupId + "/", groupGetResp1)
            .post("end:secrets/", secretPostResp);
        forceLogin(sdk).then(function () {
            sdk.create_group(groupName).then(function (group) {
                sdk.create_secret({
                    name: name,
                    type: type,
                    group_id: group.id,
                    visible_data: visibleData,
                    secrets: secrets,
                }).then(function (resp) {
                    expect(resp.name).toBe(name);
                    expect(resp.type).toBe(type);
                    sdk.decrypt_secret(resp).then(function (decrypted) {
                        expect(decrypted).toEqual(secrets);
                        done();
                    });
                });
            });
        });
    });
    it("can decrypt a secret without making api calls", function () { return __awaiter(_this, void 0, void 0, function () {
        var name, type, visibleData, secrets, resp, _a, key_ciphertext, data, group, result;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    name = "my secret";
                    type = "website";
                    visibleData = {
                        username: "myself",
                    };
                    secrets = {
                        password: "hunter4",
                    };
                    fetchMock
                        .post("end:secrets/", testResponses_1.createSecretResp);
                    return [4, forceLogin(sdk)];
                case 1:
                    _b.sent();
                    return [4, sdk.create_secret({
                            name: name,
                            type: type,
                            visible_data: visibleData,
                            secrets: secrets,
                        })];
                case 2:
                    resp = _b.sent();
                    _a = resp.secret_through_set.find(function (through) { return through.is_mine === true; }), key_ciphertext = _a.key_ciphertext, data = _a.data, group = _a.group;
                    result = sdk.offline_decrypt_secret(key_ciphertext, data);
                    expect(result.password).toBe("hunter4");
                    return [2];
            }
        });
    }); });
    it("update secret partially with patch", function (done) {
        var secretId = 1;
        var newName = "blarg";
        fetchMock
            .mock("end:secrets/" + secretId + "/", testResponses_1.updateSecretResp, { method: "PATCH" });
        forceLogin(sdk).then(function () {
            sdk.update_secret({
                id: secretId,
                name: newName,
            }).then(function (resp) {
                expect(resp.name).toBe(newName);
                done();
            });
        });
    });
    it("update secret full with put", function (done) {
        var secretId = 1;
        var newSecret = {
            password: "lol",
        };
        var putSecretResp = Object.assign({}, testResponses_1.updateSecretResp);
        var publicKey = "xRhhY/M0r65jDMRjoMVcgHEdZyj1EXGDyLGXzPYByQI=";
        var keyCiphertext = "T1vB9Omm7lLs1w4IcxgxHG1G6lZxeu61ob5uiLn6rWXET7bIiUgvRMWqvZua/JR/DxJWEfxcKJIcpBRsSIYMI7Ga4QT7kI+96cpdNEGdUJ3F7UhkH/EqbbgZeSM=";
        var password = "ypXHWRqYEuWb7eNvdTOtHRCjzqxaVH3/QJuKWRy+DyPKRFGKn1LYQzGxr1GnTOo=";
        putSecretResp.secret_through_set = [{
                id: 7,
                group: null,
                key_ciphertext: keyCiphertext,
                data: { password: password },
                public_key: publicKey,
                is_min: true,
            }];
        fetchMock
            .get("end:secrets/" + secretId + "/", testResponses_1.getSecretResp)
            .put("end:secrets/" + secretId + "/", putSecretResp);
        forceLogin(sdk).then(function () {
            sdk.update_secret({
                id: secretId,
                secrets: newSecret,
            }).then(function (resp) {
                expect(resp.secret_through_set).toEqual(putSecretResp.secret_through_set);
                done();
            });
        });
    });
    it("Create group and decrypt it's key", function (done) {
        var name = "my secret";
        var type = "website";
        var visibleData = {
            username: "myself",
        };
        var secrets = {
            password: "hunter2",
        };
        var newSecret = {
            name: name,
            type: type,
            visible_data: visibleData,
            secrets: secrets,
            group_id: null,
        };
        var groupName = "My Group";
        fetchMock
            .post("end:groups/", testResponses_1.groupsPostResp)
            .get("end:groups/51/", testResponses_1.groupGetResp);
        forceLogin(sdk).then(function () {
            sdk.create_group(groupName).then(function (createdGroup) {
                sdk.get_group(createdGroup.id).then(function (group) {
                    var keyPair = sdk._getGroupKeyPair(group);
                    expect(keyPair.privateKey).toBeTruthy();
                    done();
                });
            });
        });
    });
    xit("reset password with real backend", function (done) {
        var name = "my secret";
        var oldPassword = testPassword;
        var type = "website";
        var secrets = {
            password: "hunter2",
        };
        var newSecret = {
            name: name,
            type: type,
            visible_data: {},
            secrets: secrets,
        };
        var groupName = "My Group";
        var otherUserId = 2;
        var otherUserEmail = "noadmin@aa.aa";
        var newMasterPassword = "my great pass";
        var makeSharedSecret = function () {
            return new Promise(function (resolve, reject) {
                sdk.create_group(groupName).then(function (group) {
                    sdk.create_secret(newSecret).then(function (secret) {
                        sdk.add_group_to_secret(group.id, secret).then(function () {
                            sdk.add_user_to_group(group.id, otherUserId, otherUserEmail).then(function () {
                                sdk.get_secret(secret.id).then(function (secret2) {
                                    sdk.decrypt_secret(secret).then(function (resp) {
                                        expect(resp.password).toBe("hunter2");
                                        resolve(secret2);
                                    });
                                });
                            });
                        });
                    });
                });
            });
        };
        forceLogin(sdk, true).then(function () {
            sdk.create_secret(newSecret).then(function (secret2) {
                makeSharedSecret().then(function (secret) {
                    setTimeout(function () {
                        sdk.change_password(oldPassword, newMasterPassword).then(function () {
                            sdk = new sdk_1.default();
                            forceLogin(sdk, true, newMasterPassword).then(function () {
                                sdk.get_secret(secret2.id).then(function (s2) {
                                    sdk.decrypt_secret(s2).then(function (r2) {
                                        expect(r2.password).toBe("hunter2");
                                    });
                                });
                                sdk.get_secret(secret.id).then(function (secret3) {
                                    sdk.decrypt_secret(secret3).then(function (resp2) {
                                        expect(resp2.password).toBe("hunter2");
                                        done();
                                    });
                                });
                            });
                        });
                    }, 100);
                });
            });
        });
    }, 20000);
    xit("update secret and decrypt it with real backend", function (done) {
        var name = "my secret";
        var type = "website";
        var visibleData = {
            username: "myself",
        };
        var secrets = {
            password: "hunter2",
        };
        var newSecret = {
            name: name,
            type: type,
            visible_data: visibleData,
            secrets: secrets,
        };
        var groupName = "My Group";
        var group;
        var otherUserId = 2;
        var otherUserEmail = "noadmin@aa.aa";
        forceLogin(sdk, true).then(function () {
            sdk.create_group(groupName).then(function (resp) {
                group = resp;
            });
            sdk.create_secret(newSecret).then(function (secret) {
                sdk.decrypt_secret(secret).then(function (resp) {
                    expect(resp.password).toBe("hunter2");
                });
                var updatedSecret = Object.assign(newSecret, { id: secret.id, secrets: { password: "hunter3" } });
                sdk.update_secret(updatedSecret).then(function (updatedSecretResp) {
                    sdk.decrypt_secret(updatedSecretResp).then(function (resp) {
                        expect(resp.password).toBe("hunter3");
                        sdk.add_group_to_secret(group.id, secret.id).then(function () {
                            sdk.add_user_to_group(group.id, otherUserId, otherUserEmail).then(function () {
                                sdk.get_secret(secret.id).then(function (secret) {
                                    sdk.decrypt_secret(secret).then(function (resp) {
                                        expect(resp.password).toBe("hunter3");
                                        var editedSecret = {
                                            id: secret.id,
                                            secrets: {
                                                password: "hunter4",
                                            },
                                        };
                                        sdk.update_secret(editedSecret).then(function (secret) {
                                            sdk.decrypt_secret(secret).then(function (resp) {
                                                expect(resp.password).toBe("hunter4");
                                                done();
                                            });
                                        });
                                    });
                                });
                            });
                        });
                    });
                });
            });
        });
    });
    it("get secret", function (done) {
        var secretId = 1;
        fetchMock
            .get("end:secrets/" + secretId + "/", testResponses_1.getSecretResp);
        forceLogin(sdk).then(function () {
            sdk.get_secret(secretId).then(function (resp) {
                expect(resp.name).toBe("my secret");
                done();
            });
        });
    });
    it("get and decrypt secret", function (done) {
        var secretId = 1;
        fetchMock
            .get("end:secrets/" + secretId + "/", testResponses_1.getSecretResp);
        forceLogin(sdk).then(function () {
            sdk.get_secret(secretId).then(function (resp) {
                sdk.decrypt_secret(resp).then(function (secret) {
                    expect(secret.password).toBe("hunter4");
                    done();
                });
            });
        });
    });
    it("delete secret", function (done) {
        var secretId = 9;
        fetchMock
            .mock("end:secrets/" + secretId + "/", { status: 204 });
        forceLogin(sdk).then(function () {
            sdk.delete_secret(secretId).then(function () {
                done();
            });
        });
    });
    it("list secrets", function (done) {
        fetchMock
            .get("end:secrets/", testResponses_1.listSecretResp);
        forceLogin(sdk).then(function () {
            sdk.list_secrets().then(function (resp) {
                expect(resp[0].name).toBe("my secret");
                done();
            });
        });
    });
    it("add group to secret", function (done) {
        var secretId = 1;
        var groupId = 1;
        fetchMock
            .get("end:groups/" + groupId + "/", testResponses_1.groupDetailResp)
            .get("end:secrets/" + secretId + "/", testResponses_1.getSecretResp)
            .post("end:secrets/9/groups/", testResponses_1.secretThroughObj);
        forceLogin(sdk).then(function () {
            sdk.add_group_to_secret(groupId, secretId).then(function (resp) {
                expect(resp.is_mine).toBe(true);
                done();
            });
        });
    });
    it("remove group from secret", function (done) {
        var secretId = 1;
        var secretThroughId = 1;
        fetchMock
            .delete("end:secrets/" + secretId + "/groups/" + secretThroughId + "/", { status: 204 });
        forceLogin(sdk).then(function () {
            sdk.remove_group_from_secret(secretId, secretThroughId).then(function () {
                done();
            });
        });
    });
    it("change password", function (done) {
        var oldPassword = testPassword;
        var newPassword = "MyNewPassword";
        fetchMock
            .get("end:secrets/", [testResponses_1.getSecretResp])
            .get("end:groups/", [testResponses_1.groupResp])
            .get("end:groups/1/", testResponses_1.groupResp)
            .get("end:users/", [testResponses_1.signUpResp])
            .post("end:change-password/", { success: true });
        forceLogin(sdk).then(function () {
            sdk.change_password(oldPassword, newPassword).then(function () {
                done();
            });
        });
    });
});
//# sourceMappingURL=sdk.spec.js.map